//+------------------------------------------------------------------+
//|                                                          Complex |
//|                        Copyright 2009, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "2009, MetaQuotes Software Corp."
#property link      "http://www.mql5.com"
#property version   "2.00"

#define EURUSD	0
#define GBPUSD	1
#define EURGBP	2
#define EURJPY	3
#define USDJPY	4
#define GBPJPY	5

#property indicator_separate_window
#property indicator_buffers 50
#property indicator_plots   1

#property indicator_label1 "Complex2"
#property indicator_type1 DRAW_LINE
#property indicator_color1 clrGreen
#property indicator_style1 STYLE_SOLID
#property indicator_width1 1

//--- input parameters
input int MA_Slow=56;
input ENUM_TIMEFRAMES MA_Slow_TF=PERIOD_H1;
input int MA_Fast=9;
input ENUM_TIMEFRAMES MA_Fast_TF=PERIOD_H1;
input int nBars=500;
input double kUSD=1;
input double kGBP=1;
input double kJPY=1;
input double kEUR=1;

//------------------------------------------------------------------	class CSeries
class CSeries
{
public:
	double open[], high[], low[], close[]; //      
	int ho, hh, hl, hc; //  

public:
	CSeries() { ho=INVALID_HANDLE; hh=INVALID_HANDLE; hl=INVALID_HANDLE; hc=INVALID_HANDLE; };
	~CSeries() { IndicatorRelease(ho); IndicatorRelease(hh); IndicatorRelease(hl); IndicatorRelease(hc); }
public:
	void SetIndexBuf(int istart); //   
	void CreateHandle(string name, ENUM_TIMEFRAMES TF, int MA); //  
	int CopyBuf(int n); //   
};
//------------------------------------------------------------------	SetIndexBuf
void CSeries::SetIndexBuf(int istart) //   
{
	SetIndexBuffer(istart+0, open,INDICATOR_CALCULATIONS);
	SetIndexBuffer(istart+1, high, INDICATOR_CALCULATIONS);
	SetIndexBuffer(istart+2, low, INDICATOR_CALCULATIONS);
	SetIndexBuffer(istart+3, close, INDICATOR_CALCULATIONS);
	ArraySetAsSeries(open, true);
	ArraySetAsSeries(high, true);
	ArraySetAsSeries(low, true);
	ArraySetAsSeries(close, true);
}
//------------------------------------------------------------------	CreateHandle
void CSeries::CreateHandle(string name, ENUM_TIMEFRAMES TF, int MA) //  
{
	ho=iMA(name, TF, MA, 0, MODE_LWMA, PRICE_OPEN);
	hh=iMA(name, TF, MA, 0, MODE_LWMA,PRICE_HIGH);
	hl=iMA(name, TF, MA, 0, MODE_LWMA, PRICE_LOW);
	hc=iMA(name, TF, MA, 0, MODE_LWMA, PRICE_CLOSE);
}
//------------------------------------------------------------------	CopyBuf
int CSeries::CopyBuf(int n) //   
{
	int c=0; //   ,      
	if ((c=CopyBuffer(ho, 0, 0, n, open))<n) return(-1);
	if ((c=CopyBuffer(hh, 0, 0, n, high))<n) return(-1);
	if ((c=CopyBuffer(hl, 0, 0, n, low))<n) return(-1);
	if ((c=CopyBuffer(hc, 0, 0, n, close))<n) return(-1);
	return(n);
}

//------------------------------------------------------------------	class CPair
class CPair
{
public:
	string m_name; //  
	CSeries F, S; // 
	void SetIndexBuf(int istart) { S.SetIndexBuf(istart); F.SetIndexBuf(istart+4); }
	void CreateHandle(ENUM_TIMEFRAMES sTF, int sMA, ENUM_TIMEFRAMES fTF, int fMA) { S.CreateHandle(m_name, sTF, sMA); F.CreateHandle(m_name, fTF, fMA); }
	int CopyBuf(int n) { if (S.CopyBuf(n)<0) return(-1); if (F.CopyBuf(n)<0) return(-1); return(n); }
public:
	CPair() { };
	~CPair() { };
public:
};

double Mediana[];
CPair Pair[6];

//------------------------------------------------------------------	OnInit
int OnInit()
{
	SetIndexBuffer(0, Mediana, INDICATOR_DATA);
	ArraySetAsSeries(Mediana, true);
	//   
	Pair[EURUSD].m_name="EURUSD"; Pair[GBPUSD].m_name="GBPUSD"; Pair[EURGBP].m_name="EURGBP";
	Pair[EURJPY].m_name="EURJPY"; Pair[USDJPY].m_name="USDJPY"; Pair[GBPJPY].m_name="GBPJPY";
	for (int i=0; i<6; i++) //     
	{
		Pair[i].SetIndexBuf((i+1)+8*i);
		Pair[i].CreateHandle(MA_Slow_TF, MA_Slow, MA_Fast_TF, MA_Fast);
	}   
	string shortname;
	StringConcatenate(shortname, "Complex(",MA_Slow,",",MA_Fast,")");
	PlotIndexSetString(0, PLOT_LABEL, shortname); //     DataWindow
	IndicatorSetString(INDICATOR_SHORTNAME, shortname); //          
	IndicatorSetInteger(INDICATOR_DIGITS, 5); //    
	return(0);
}
//------------------------------------------------------------------	OnCalculate
int OnCalculate(const int rates_total, const int prev_calculated,
                const datetime& time[], const double& open[], const double& high[], const double& low[], const double& close[],
                const long& tick_volume[], const long& volume[], const int& spread[])
{

	for (int i=0; i<6; i++) if (Pair[i].CopyBuf(nBars)<0) { Print("Wait Data for "+Pair[i].m_name); return(0); } //       //   
	
	int limit=rates_total-prev_calculated; if (limit>=rates_total) limit=rates_total-1; //   
	if (limit>nBars) limit=nBars; //     
	
	for(int i=0; i<limit; i++) //       
	{
		Mediana[i]=EMPTY_VALUE;
		double O=0, H=0, L=0, C=0;
		if (Symbol()=="EURUSD")
		{
			O=EUR(PRICE_OPEN, i)-USD(PRICE_OPEN, i); H=EUR(PRICE_HIGH,i)-USD(PRICE_HIGH, i);
			L=EUR(PRICE_LOW, i)-USD(PRICE_LOW, i); C=EUR(PRICE_CLOSE, i)-USD(PRICE_CLOSE, i);
		}
		if (Symbol()=="EURGBP")
		{
			O=EUR(PRICE_OPEN, i)-GBP(PRICE_OPEN, i); H=EUR(PRICE_HIGH, i)-GBP(PRICE_HIGH, i);
			L=EUR(PRICE_LOW, i)-GBP(PRICE_LOW, i); C=EUR(PRICE_CLOSE, i)-GBP(PRICE_CLOSE, i);
		}
		
		if (Symbol()=="EURJPY")
		{
			O=EUR(PRICE_OPEN, i)-JPY(PRICE_OPEN, i); H=EUR(PRICE_HIGH, i)-JPY(PRICE_HIGH, i);
			L=EUR(PRICE_LOW, i)-JPY(PRICE_LOW, i); C=EUR(PRICE_CLOSE, i)-JPY(PRICE_CLOSE, i);
		}
		if (Symbol()=="GBPUSD")
		{
			O=GBP(PRICE_OPEN, i)-USD(PRICE_OPEN, i); H=GBP(PRICE_HIGH, i)-USD(PRICE_HIGH, i);
			L=GBP(PRICE_LOW, i)-USD(PRICE_LOW, i); C=GBP(PRICE_CLOSE, i)-USD(PRICE_CLOSE, i);
		}
		
		if (Symbol()=="GBPJPY")
		{
			O=GBP(PRICE_OPEN, i)-JPY(PRICE_OPEN, i); H=GBP(PRICE_HIGH, i)-JPY(PRICE_HIGH, i);
			L=GBP(PRICE_LOW, i)-JPY(PRICE_LOW, i); C=GBP(PRICE_CLOSE, i)-JPY(PRICE_CLOSE, i);
		}
		
		if (Symbol()=="USDJPY")
		{
			O=USD(PRICE_OPEN, i)-JPY(PRICE_OPEN, i); H=USD(PRICE_HIGH, i)-JPY(PRICE_HIGH, i);
			L=USD(PRICE_LOW, i)-JPY(PRICE_LOW, i); C=USD(PRICE_CLOSE, i)-JPY(PRICE_CLOSE, i);
		}
		Mediana[i]=(O+H+L+C)/4;
	}
	return(rates_total);
}
//------------------------------------------------------------------	USD
double USD(ENUM_APPLIED_PRICE price, int i)
{
	CSeries *eurS=GetPointer(Pair[EURUSD].S), *gbpS=GetPointer(Pair[GBPUSD].S), *jpyS=GetPointer(Pair[USDJPY].S);
	CSeries *eurF=GetPointer(Pair[EURUSD].F), *gbpF=GetPointer(Pair[GBPUSD].F), *jpyF=GetPointer(Pair[USDJPY].F);
	if (price==PRICE_OPEN) return((eurS.open[i]-eurF.open[i])*10000*kEUR+(gbpS.open[i]-gbpF.open[i])*10000*kGBP+(jpyF.open[i]-jpyS.open[i])*100*kJPY);
	if (price==PRICE_HIGH) return((eurS.high[i]-eurF.high[i])*10000*kEUR+(gbpS.high[i]-gbpF.high[i])*10000*kGBP+(jpyF.high[i]-jpyS.high[i])*100*kJPY);
	if (price==PRICE_LOW) return((eurS.low[i]-eurF.low[i])*10000*kEUR+(gbpS.low[i]-gbpF.low[i])*10000*kGBP+(jpyF.low[i]-jpyS.low[i])*100*kJPY);
	if (price==PRICE_CLOSE) return((eurS.close[i]-eurF.close[i])*10000*kEUR+(gbpS.close[i]-gbpF.close[i])*10000*kGBP+(jpyF.close[i]-jpyS.close[i])*100*kJPY);
	return(0);
}
//------------------------------------------------------------------	EUR
double EUR(ENUM_APPLIED_PRICE price, int i)
{
	CSeries *usdS=GetPointer(Pair[EURUSD].S), *gbpS=GetPointer(Pair[EURGBP].S), *jpyS=GetPointer(Pair[EURJPY].S);
	CSeries *usdF=GetPointer(Pair[EURUSD].F), *gbpF=GetPointer(Pair[EURGBP].F), *jpyF=GetPointer(Pair[EURJPY].F);
	if (price==PRICE_OPEN) return((usdF.open[i]-usdS.open[i])*10000*kUSD+(gbpF.open[i]-gbpS.open[i])*10000*kGBP+(jpyF.open[i]-jpyS.open[i])*100*kJPY);
	if (price==PRICE_HIGH) return((usdF.high[i]-usdS.high[i])*10000*kUSD+(gbpF.high[i]-gbpS.high[i])*10000*kGBP+(jpyF.high[i]-jpyS.high[i])*100*kJPY);
	if (price==PRICE_LOW) return((usdF.low[i]-usdS.low[i])*10000*kUSD+(gbpF.low[i]-gbpS.low[i])*10000*kGBP+(jpyF.low[i]-jpyS.low[i])*100*kJPY);
	if (price==PRICE_CLOSE) return((usdF.close[i]-usdS.close[i])*10000*kUSD+(gbpF.close[i]-gbpS.close[i])*10000*kGBP+(jpyF.close[i]-jpyS.close[i])*100*kJPY);
	return(0);
}
//------------------------------------------------------------------	GBP
double GBP(ENUM_APPLIED_PRICE price, int i)
{
	CSeries *usdS=GetPointer(Pair[GBPUSD].S), *eurS=GetPointer(Pair[EURGBP].S), *jpyS=GetPointer(Pair[GBPJPY].S);
	CSeries *usdF=GetPointer(Pair[GBPUSD].F), *eurF=GetPointer(Pair[EURGBP].F), *jpyF=GetPointer(Pair[GBPJPY].F);
	if (price==PRICE_OPEN) return((usdF.open[i]-usdS.open[i])*10000*kUSD+(eurS.open[i]-eurF.open[i])*10000*kEUR+(jpyF.open[i]-jpyS.open[i])*100*kJPY);
	if (price==PRICE_HIGH) return((usdF.high[i]-usdS.high[i])*10000*kUSD+(eurS.high[i]-eurF.high[i])*10000*kEUR+(jpyF.high[i]-jpyS.high[i])*100*kJPY);
	if (price==PRICE_LOW) return((usdF.low[i]-usdS.low[i])*10000*kUSD+(eurS.low[i]-eurF.low[i])*10000*kEUR+(jpyF.low[i]-jpyS.low[i])*100*kJPY);
	if (price==PRICE_CLOSE) return((usdF.close[i]-usdS.close[i])*10000*kUSD+(eurS.close[i]-eurF.close[i])*10000*kEUR+(jpyF.close[i]-jpyS.close[i])*100*kJPY);
	return(0);
}      
//------------------------------------------------------------------	JPY
double JPY(ENUM_APPLIED_PRICE price, int i)
{
	CSeries *usdS=GetPointer(Pair[USDJPY].S), *eurS=GetPointer(Pair[EURJPY].S), *gbpS=GetPointer(Pair[GBPJPY].S);
	CSeries *usdF=GetPointer(Pair[USDJPY].F), *eurF=GetPointer(Pair[EURJPY].F), *gbpF=GetPointer(Pair[GBPJPY].F);
	if (price==PRICE_OPEN) return((usdS.open[i]-usdF.open[i])*100*kUSD+(eurS.open[i]-eurF.open[i])*100*kEUR+(gbpS.open[i]-gbpF.open[i])*100*kGBP);
	if (price==PRICE_HIGH) return((usdS.high[i]-usdF.high[i])*100*kUSD+(eurS.high[i]-eurF.high[i])*100*kEUR+(gbpS.high[i]-gbpF.high[i])*100*kGBP);
	if (price==PRICE_LOW) return((usdS.low[i]-usdF.low[i])*100*kUSD+(eurS.low[i]-eurF.low[i])*100*kEUR+(gbpS.low[i]-gbpF.low[i])*100*kGBP);
	if (price==PRICE_CLOSE) return((usdS.close[i]-usdF.close[i])*100*kUSD+(eurS.close[i]-eurF.close[i])*100*kEUR+(gbpS.close[i]-gbpF.close[i])*100*kGBP);
	return(0);
}  